@@ -27,7 +27,9 @@
 //   http://dx.doi.org/10.1016/j.chaos.2016.05.003
 //
 // =======================================================================
-// Implementation by Konstantin Savvidy - Copyright 2004-2017
+// Implementation by Konstantin Savvidy - Copyright 2004-2023
+// July 2023 - Updated class structure upon suggestions from Marco Barbone
+// September 2023 - fix (re-)initialization from Gabriele Cosmo
 // =======================================================================
 
 #include "CLHEP/Random/defs.h"
@@ -36,8 +38,12 @@
 #include "CLHEP/Random/engineIDulong.h"
 #include "CLHEP/Utility/atomic_int.h"
 
-#include <string.h>        // for strcmp
+#include <atomic>
 #include <cmath>
+#include <algorithm>
+#include <iostream>
+#include <string.h>        // for strcmp
+#include <vector>
 
 const unsigned long MASK32=0xffffffff;
 
@@ -48,7 +54,7 @@ namespace {
   CLHEP_ATOMIC_INT_TYPE numberOfEngines(0);
 }
 
-static const int MarkerLen = 64; // Enough room to hold a begin or end marker. 
+static const int MarkerLen = 64; // Enough room to hold a begin or end marker.
 
 MixMaxRng::MixMaxRng()
 : HepRandomEngine()
@@ -70,16 +76,16 @@ MixMaxRng::MixMaxRng(std::istream& is)
    get(is);
 }
 
-MixMaxRng::~MixMaxRng() 
+MixMaxRng::~MixMaxRng()
 {
 }
 
 MixMaxRng::MixMaxRng(const MixMaxRng& rng)
   : HepRandomEngine(rng)
 {
-   S.V = rng.S.V;
-   S.sumtot= rng.S.sumtot;
-   S.counter= rng.S.counter;
+   std::copy(rng.V, rng.V+N, V);
+   sumtot= rng.sumtot;
+   counter= rng.counter;
 }
 
 MixMaxRng& MixMaxRng::operator=(const MixMaxRng& rng)
@@ -92,9 +98,9 @@ MixMaxRng& MixMaxRng::operator=(const MixMaxRng& rng)
    //
    HepRandomEngine::operator=(rng);
 
-   S.V = rng.S.V;
-   S.sumtot= rng.S.sumtot;
-   S.counter= rng.S.counter;
+   std::copy(rng.V, rng.V+N, V);
+   sumtot= rng.sumtot;
+   counter= rng.counter;
 
    return *this;
 }
@@ -108,23 +114,17 @@ void MixMaxRng::saveStatus( const char filename[] ) const
      int j;
      fprintf(fh, "mixmax state, file version 1.0\n" );
      fprintf(fh, "N=%u; V[N]={", rng_get_N() );
-     for (j=0; (j< (rng_get_N()-1) ); j++) {
-         fprintf(fh, "%llu, ", S.V[j] );
+     for (j=0; (j< (rng_get_N()-1) ); ++j) {
+         fprintf(fh, "%llu, ", (unsigned long long)V[j] );
      }
-     fprintf(fh, "%llu", S.V[rng_get_N()-1] );
+     fprintf(fh, "%llu", (unsigned long long)V[rng_get_N()-1] );
      fprintf(fh, "}; " );
-     fprintf(fh, "counter=%u; ", S.counter );
-     fprintf(fh, "sumtot=%llu;\n", S.sumtot );
+     fprintf(fh, "counter=%u; ", counter );
+     fprintf(fh, "sumtot=%llu;\n", (unsigned long long)sumtot );
      fclose(fh);
    }
 }
 
-#define MIXMAX_ARRAY_INDEX_OUT_OF_BOUNDS   0xFF01
-#define MIXMAX_SEED_WAS_ZERO               0xFF02
-#define MIXMAX_ERROR_READING_STATE_FILE    0xFF03
-#define MIXMAX_ERROR_READING_STATE_COUNTER       0xFF04
-#define MIXMAX_ERROR_READING_STATE_CHECKSUM      0xFF05
-
 void MixMaxRng::restoreStatus( const char filename[] )
 {
    // a function for reading the state from a file
@@ -140,77 +140,70 @@ void MixMaxRng::restoreStatus( const char filename[] )
    else
    {
       fprintf(stderr, "mixmax -> read_state: error reading file %s\n", filename);
-      exit(MIXMAX_ERROR_READING_STATE_FILE);
+      throw std::runtime_error("Error in reading state file");
    }
     
    myuint_t vecVal;
    //printf("mixmax -> read_state: starting to read state from file\n");
-   if (!fscanf(fin, "%llu", &S.V[0]) )
+   if (!fscanf(fin, "%llu", (unsigned long long*) &V[0]) )
    {
      fprintf(stderr, "mixmax -> read_state: error reading file %s\n", filename);
-     exit(MIXMAX_ERROR_READING_STATE_FILE);
+     throw std::runtime_error("Error in reading state file");
    }
 
-   int i;
-   for( i = 1; i < rng_get_N(); i++)
+   for (int i = 1; i < rng_get_N(); ++i)
    {
-     if (!fscanf(fin, ", %llu", &vecVal) )
+     if (!fscanf(fin, ", %llu", (unsigned long long*) &vecVal) )
      {
        fprintf(stderr, "mixmax -> read_state: error reading vector component i=%d from file %s\n", i, filename);
-       exit(MIXMAX_ERROR_READING_STATE_FILE);
+       throw std::runtime_error("Error in reading state file");
      }
-     if(  vecVal <= MixMaxRng::M61 )
+     if( vecVal <= MixMaxRng::M61 )
      {
-       S.V[i] = vecVal;
+       V[i] = vecVal;
      }
      else
      {
        fprintf(stderr, "mixmax -> read_state: Invalid state vector value= %llu"
                " ( must be less than %llu ) "
                " obtained from reading file %s\n"
-               , vecVal, MixMaxRng::M61, filename);
+               , (unsigned long long)vecVal, (unsigned long long)MixMaxRng::M61, filename);
      }
    }
     
-   int counter;
-   if (!fscanf( fin, "}; counter=%i; ", &counter))
+   int incounter;
+   if (!fscanf( fin, "}; counter=%i; ", &incounter))
    {
      fprintf(stderr, "mixmax -> read_state: error reading counter from file %s\n", filename);
-     exit(MIXMAX_ERROR_READING_STATE_FILE);
+     throw std::runtime_error("Error in reading state file");
    }
-   if( counter <= rng_get_N() )
+   if( incounter <= rng_get_N() )
    {
-     S.counter= counter;
+     counter = incounter;
    }
    else
    {
      fprintf(stderr, "mixmax -> read_state: Invalid counter = %d"
              "  Must be 0 <= counter < %u\n" , counter, rng_get_N());
      print_state();
-     exit(MIXMAX_ERROR_READING_STATE_COUNTER);
+     throw std::runtime_error("Error in reading state counter");
    }
    precalc();
-   myuint_t sumtot;
-   if (!fscanf( fin, "sumtot=%llu\n", &sumtot))
+   myuint_t insumtot;
+   if (!fscanf( fin, "sumtot=%llu\n", (unsigned long long*) &insumtot))
    {
      fprintf(stderr, "mixmax -> read_state: error reading checksum from file %s\n", filename);
-     exit(MIXMAX_ERROR_READING_STATE_FILE);
+     throw std::runtime_error("Error in reading state file");
    }
 
-   if (S.sumtot != sumtot)
+   if (sumtot != insumtot)
    {
      fprintf(stderr, "mixmax -> checksum error while reading state from file %s - corrupted?\n", filename);
-     exit(MIXMAX_ERROR_READING_STATE_CHECKSUM);
+     throw std::runtime_error("Error in reading state checksum");
    }
    fclose(fin);
 }
 
-#undef MIXMAX_ARRAY_INDEX_OUT_OF_BOUNDS
-#undef MIXMAX_SEED_WAS_ZERO
-#undef MIXMAX_ERROR_READING_STATE_FILE
-#undef MIXMAX_ERROR_READING_STATE_COUNTER
-#undef MIXMAX_ERROR_READING_STATE_CHECKSUM
-
 void MixMaxRng::showStatus() const
 {
    std::cout << std::endl;
@@ -221,37 +214,30 @@ void MixMaxRng::showStatus() const
    std::cout << "---------------------------------------" << std::endl;
 }
 
-void MixMaxRng::setSeed(long longSeed, int /* extraSeed */)
-{
-   //seed_uniquestream(0,0,0,longSeed);
-   theSeed = longSeed;
-   seed_spbox(longSeed);
-}
-
 //  Preferred Seeding method
-//  the values of 'Seeds' must be valid 32-bit integers 
+//  the values of 'Seeds' must be valid 32-bit integers
 //  Higher order bits will be ignored!!
 
 void MixMaxRng::setSeeds(const long* Seeds, int seedNum)
 {
-   unsigned long seed0, seed1= 0, seed2= 0, seed3= 0;
+   myID_t seed0, seed1= 0, seed2= 0, seed3= 0;
 
    if( seedNum < 1 ) {  // Assuming at least 2 seeds in vector...
-       seed0= static_cast<unsigned long>(Seeds[0]) & MASK32;
-       seed1= static_cast<unsigned long>(Seeds[1]) & MASK32;
-   } 
+       seed0= static_cast<myID_t>(Seeds[0]) & MASK32;
+       seed1= static_cast<myID_t>(Seeds[1]) & MASK32;
+   }
    else
    {
      if( seedNum < 4 ) {
-       seed0= static_cast<unsigned long>(Seeds[0]) & MASK32;
-       if( seedNum > 1){ seed1= static_cast<unsigned long>(Seeds[1]) & MASK32; }
-       if( seedNum > 2){ seed2= static_cast<unsigned long>(Seeds[2]) & MASK32; }
+       seed0= static_cast<myID_t>(Seeds[0]) & MASK32;
+       if( seedNum > 1){ seed1= static_cast<myID_t>(Seeds[1]) & MASK32; }
+       if( seedNum > 2){ seed2= static_cast<myID_t>(Seeds[2]) & MASK32; }
      }
      if( seedNum >= 4 ) {
-       seed0= static_cast<unsigned long>(Seeds[0]) & MASK32;
-       seed1= static_cast<unsigned long>(Seeds[1]) & MASK32;
-       seed2= static_cast<unsigned long>(Seeds[2]) & MASK32;
-       seed3= static_cast<unsigned long>(Seeds[3]) & MASK32;
+       seed0= static_cast<myID_t>(Seeds[0]) & MASK32;
+       seed1= static_cast<myID_t>(Seeds[1]) & MASK32;
+       seed2= static_cast<myID_t>(Seeds[2]) & MASK32;
+       seed3= static_cast<myID_t>(Seeds[3]) & MASK32;
      }
    }
    theSeed = Seeds[0];
@@ -269,126 +255,44 @@ constexpr int MixMaxRng::rng_get_N()
    return N;
 }
 
-constexpr long long int MixMaxRng::rng_get_SPECIAL()
-{
-   return SPECIAL;
-}
-
-constexpr int MixMaxRng::rng_get_SPECIALMUL()
-{
-   return SPECIALMUL;
-}
-
-double MixMaxRng::generate(int i)
-{
-   S.counter++;
-#if defined(__clang__) || defined(__llvm__)
-   return INV_M61*static_cast<double>(S.V[i]);
-#elif defined(__GNUC__) && (__GNUC__ < 7) && (!defined(__ICC)) && defined(__x86_64__) && defined(__SSE2_MATH__)
-   int64_t Z=S.V[i];
-   double F=0.0;
-   //#warning Using the inline assembler
-    /* using SSE inline assemly to zero the xmm register, just before int64 -> double conversion,
-       not necessary in GCC-5 or better, but huge penalty on earlier compilers
-    */
-   __asm__ __volatile__(  "pxor %0, %0;"
-                          "cvtsi2sdq %1, %0;"
-                          :"=x"(F)
-                          :"r"(Z)
-                       );
-   return F*INV_M61;
-#else
-  //#warning other method
-   return convert1double(S.V[i]); //get_next_float_packbits();
-#endif
-}
-
-double MixMaxRng::iterate()
-{
-   myuint_t* Y=S.V.data();
-   myuint_t  tempP, tempV;
-   Y[0] = ( tempV = S.sumtot);
-   myuint_t sumtot = Y[0], ovflow = 0; // will keep a running sum of all new elements
-   tempP = 0;              // will keep a partial sum of all old elements
-   myuint_t tempPO;
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[1] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[1]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[2] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[2]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[3] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[3]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[4] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[4]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[5] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[5]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[6] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[6]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[7] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[7]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[8] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[8]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[9] ); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[9]  = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[10]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[10] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[11]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[11] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[12]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[12] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[13]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[13] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[14]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[14] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[15]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[15] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   tempPO = MULWU(tempP); tempP = modadd(tempP, Y[16]); tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); Y[16] = tempV; sumtot += tempV; if (sumtot < tempV) {ovflow++;};
-   S.sumtot = MIXMAX_MOD_MERSENNE(MIXMAX_MOD_MERSENNE(sumtot) + (ovflow <<3 ));
-
-   S.counter=2;
-   return double(S.V[1])*INV_M61;
-}
-
 void MixMaxRng::flatArray(const int size, double* vect )
 {
-   // fill_array( S, size, arrayDbl );
    for (int i=0; i<size; ++i) { vect[i] = flat(); }
 }
 
-MixMaxRng::operator double()
-{
-  return flat();
-}
-
-MixMaxRng::operator float()
-{
-  return float( flat() );
-}
-
-MixMaxRng::operator unsigned int()
-{
-   return static_cast<unsigned int>(get_next());
-   // clhep_get_next returns a 64-bit integer, of which the lower 61 bits
-   // are random and upper 3 bits are zero
-}
-
 std::ostream & MixMaxRng::put ( std::ostream& os ) const
 {
    char beginMarker[] = "MixMaxRng-begin";
    char endMarker[]   = "MixMaxRng-end";
 
-   int pr = os.precision(24);
+   long pr = os.precision(24);
    os << beginMarker << " ";
    os << theSeed << "\n";
    for (int i=0; i<rng_get_N(); ++i) {
-      os <<  S.V[i] << "\n";
+      os <<  V[i] << "\n";
    }
-   os << S.counter << "\n";
-   os << S.sumtot << "\n";
+   os << counter << "\n";
+   os << sumtot << "\n";
    os << endMarker << "\n";
    os.precision(pr);
-   return os;  
+   return os;
 }
 
 std::vector<unsigned long> MixMaxRng::put () const
 {
-   std::vector<unsigned long> v;
-   v.push_back (engineIDulong<MixMaxRng>());
+   std::vector<unsigned long> vec;
+   vec.push_back (engineIDulong<MixMaxRng>());
    for (int i=0; i<rng_get_N(); ++i)
    {
-     v.push_back(static_cast<unsigned long>(S.V[i] & MASK32));
+     vec.push_back(static_cast<unsigned long>(V[i] & MASK32));
        // little-ended order on all platforms
-     v.push_back(static_cast<unsigned long>(S.V[i] >> 32  ));
+     vec.push_back(static_cast<unsigned long>(V[i] >> 32  ));
        // pack uint64 into a data structure which is 32-bit on some platforms
    }
-   v.push_back(static_cast<unsigned long>(S.counter));
-   v.push_back(static_cast<unsigned long>(S.sumtot & MASK32));
-   v.push_back(static_cast<unsigned long>(S.sumtot >> 32));
-   return v;
+   vec.push_back(static_cast<unsigned long>(counter));
+   vec.push_back(static_cast<unsigned long>(sumtot & MASK32));
+   vec.push_back(static_cast<unsigned long>(sumtot >> 32));
+   return vec;
 }
 
 std::istream & MixMaxRng::get  ( std::istream& is)
@@ -396,7 +300,7 @@ std::istream & MixMaxRng::get  ( std::istream& is)
    char beginMarker [MarkerLen];
    is >> std::ws;
    is.width(MarkerLen);  // causes the next read to the char* to be <=
-                         // that many bytes, INCLUDING A TERMINATION \0 
+                         // that many bytes, INCLUDING A TERMINATION \0
                          // (Stroustrup, section 21.3.2)
    is >> beginMarker;
    if (strcmp(beginMarker,"MixMaxRng-begin")) {
@@ -410,16 +314,16 @@ std::istream & MixMaxRng::get  ( std::istream& is)
 }
 
 std::string MixMaxRng::beginTag ()
-{ 
-   return "MixMaxRng-begin"; 
+{
+   return "MixMaxRng-begin";
 }
 
 std::istream &  MixMaxRng::getState ( std::istream& is )
 {
    char endMarker[MarkerLen];
    is >> theSeed;
-   for (int i=0; i<rng_get_N(); ++i)  is >> S.V[i];
-   is >> S.counter;
+   for (int i=0; i<rng_get_N(); ++i)  is >> V[i];
+   is >> counter;
    myuint_t checksum;
    is >> checksum;
    is >> std::ws;
@@ -431,14 +335,14 @@ std::istream &  MixMaxRng::getState ( std::istream& is )
                  << "\nInput stream is probably mispositioned now.\n";
        return is;
    }
-   if ( S.counter < 0 || S.counter > rng_get_N() ) {
+   if ( counter < 0 || counter > rng_get_N() ) {
        std::cerr << "\nMixMaxRng::getState(): "
                  << "vector read wrong value of counter from file!"
                  << "\nInput stream is probably mispositioned now.\n";
        return is;
    }
    precalc();
-   if ( checksum != S.sumtot) {
+   if ( checksum != sumtot) {
        std::cerr << "\nMixMaxRng::getState(): "
                  << "checksum disagrees with value stored in file!"
                  << "\nInput stream is probably mispositioned now.\n";
@@ -447,31 +351,32 @@ std::istream &  MixMaxRng::getState ( std::istream& is )
    return is;
 }
 
-bool MixMaxRng::get (const std::vector<unsigned long> & v)
+bool MixMaxRng::get (const std::vector<unsigned long> & vec)
 {
-   if ((v[0] & 0xffffffffUL) != engineIDulong<MixMaxRng>()) {
-     std::cerr << 
+   if ((vec[0] & 0xffffffffUL) != engineIDulong<MixMaxRng>())
+   {
+     std::cerr <<
         "\nMixMaxRng::get(): vector has wrong ID word - state unchanged\n";
      return false;
    }
-   return getState(v);
+   return getState(vec);
 }
 
-bool MixMaxRng::getState (const std::vector<unsigned long> & v)
+bool MixMaxRng::getState (const std::vector<unsigned long> & vec)
 {
-   if (v.size() != VECTOR_STATE_SIZE ) {
+   if (vec.size() != VECTOR_STATE_SIZE ) {
      std::cerr <<
         "\nMixMaxRng::getState(): vector has wrong length - state unchanged\n";
      return false;
    }
    for (int i=1; i<2*rng_get_N() ; i=i+2) {
-     S.V[i/2]= ( (v[i] & MASK32) | ( (myuint_t)(v[i+1]) << 32 ) );
+     V[i/2]= ( (vec[i] & MASK32) | ( (myuint_t)(vec[i+1]) << 32 ) );
      // unpack from a data structure which is 32-bit on some platforms
    }
-   S.counter = v[2*rng_get_N()+1];
+   counter = (int)vec[2*rng_get_N()+1];
    precalc();
-   if ( ( (v[2*rng_get_N()+2] & MASK32)
-        | ( (myuint_t)(v[2*rng_get_N()+3]) << 32 ) ) != S.sumtot) {
+   if ( ( (vec[2*rng_get_N()+2] & MASK32)
+        | ( (myuint_t)(vec[2*rng_get_N()+3]) << 32 ) ) != sumtot) {
      std::cerr << "\nMixMaxRng::getState(): vector has wrong checksum!"
                << "\nInput vector is probably mispositioned now.\n";
      return false;
@@ -479,139 +384,84 @@ bool MixMaxRng::getState (const std::vector<unsigned long> & v)
    return true;
 }
 
-myuint_t MixMaxRng ::MOD_MULSPEC(myuint_t k)
-{
-   switch (N)
-   {
-      case 17:
-          return 0;
-          break;
-      case 8:
-          return 0;
-          break;
-      case 240:
-          return fmodmulM61( 0, SPECIAL , (k) );
-          break;
-      default:
-          std::cerr << "MIXMAX ERROR: " << "Disallowed value of parameter N\n";
-          std::terminate();
-          break;
-   }
-}
-
-myuint_t MixMaxRng::MULWU (myuint_t k)
-{
-   return (( (k)<<(SPECIALMUL) & M61) ^ ( (k) >> (BITS-SPECIALMUL))  );
-}
-
 myuint_t MixMaxRng::iterate_raw_vec(myuint_t* Y, myuint_t sumtotOld)
 {
    // operates with a raw vector, uses known sum of elements of Y
-   int i;
             
    myuint_t  tempP, tempV;
    Y[0] = ( tempV = sumtotOld);
-   myuint_t sumtot = Y[0], ovflow = 0; // will keep a running sum of all new elements
+   myuint_t insumtot = Y[0], ovflow = 0; // will keep a running sum of all new elements
    tempP = 0;              // will keep a partial sum of all old elements
-   for (i=1; (i<N); i++)
+   for (int i=1; (i<N); ++i)
    {
      myuint_t tempPO = MULWU(tempP);
      tempP = modadd(tempP, Y[i]);
      tempV = MIXMAX_MOD_MERSENNE(tempV+tempP+tempPO); // new Y[i] = old Y[i] + old partial * m
      Y[i] = tempV;
-     sumtot += tempV; if (sumtot < tempV) {ovflow++;}
+     insumtot += tempV; if (insumtot < tempV) {++ovflow;}
    }
-   return MIXMAX_MOD_MERSENNE(MIXMAX_MOD_MERSENNE(sumtot) + (ovflow <<3 ));
+   return MIXMAX_MOD_MERSENNE(MIXMAX_MOD_MERSENNE(insumtot) + (ovflow <<3 ));
 }
         
 myuint_t MixMaxRng::get_next()
 {
-   int i;
-   i=S.counter;
+   int i = counter;
             
    if ((i<=(N-1)) )
    {
-     S.counter++;
-     return S.V[i];
+     ++counter;
+     return V[i];
    }
    else
    {
-     S.sumtot = iterate_raw_vec(S.V.data(), S.sumtot);
-     S.counter=2;
-     return S.V[1];
+     sumtot = iterate_raw_vec(V, sumtot);
+     counter=2;
+     return V[1];
    }
 }
 
 myuint_t MixMaxRng::precalc()
 {
-   int i;
-   myuint_t temp;
-   temp = 0;
-   for (i=0; i < N; i++){
-     temp = MIXMAX_MOD_MERSENNE(temp + S.V[i]);
-   }
-   S.sumtot = temp;
+   myuint_t temp = 0;
+   for (int i=0; i < N; ++i) { temp = MIXMAX_MOD_MERSENNE(temp + V[i]); }
+   sumtot = temp;
    return temp;
 }
-            
-double MixMaxRng::get_next_float_packbits()
-{
-   myuint_t Z=get_next();
-   return convert1double(Z);
-}
         
-void MixMaxRng::seed_vielbein(unsigned int index)
+void MixMaxRng::state_init()
 {
-   int i;
-   if (index<N)
-   {
-     for (i=0; i < N; i++){
-       S.V[i] = 0;
-     }
-     S.V[index] = 1;
-   }
-   else
-   {
-     std::terminate();
-   }
-   S.counter = N;  // set the counter to N if iteration should happen right away
-   S.sumtot = 1;
+   for (int i=1; i < N; ++i) { V[i] = 0; }
+   V[0] = 1;
+   counter = N;  // set the counter to N if iteration should happen right away
+   sumtot = 1;
 }
-        
-#define MIXMAX_SEED_WAS_ZERO 0xFF02
 
 void MixMaxRng::seed_spbox(myuint_t seed)
 {
    // a 64-bit LCG from Knuth line 26, in combination with a bit swap is used to seed
 
-   const myuint_t MULT64=6364136223846793005ULL;
-   int i;
-            
-   myuint_t sumtot=0,ovflow=0;
    if (seed == 0)
-   {
-     fprintf(stderr, " try seeding with nonzero seed next time!\n");
-     exit(MIXMAX_SEED_WAS_ZERO);
-   }
+     throw std::runtime_error("try seeding with nonzero seed next time");
+
+   const myuint_t MULT64=6364136223846793005ULL;
+   sumtot = 0;
             
    myuint_t l = seed;
             
-   for (i=0; i < N; i++){
+   for (int i=0; i < N; ++i)
+   {
      l*=MULT64; l = (l << 32) ^ (l>>32);
-     S.V[i] = l & M61;
-     sumtot += S.V[(i)]; if (sumtot < S.V[(i)]) {ovflow++;}
+     V[i] = l & M61;
+     sumtot = MIXMAX_MOD_MERSENNE(sumtot + V[(i)]);
    }
-   S.counter = N;  // set the counter to N if iteration should happen right away
-   S.sumtot = MIXMAX_MOD_MERSENNE(MIXMAX_MOD_MERSENNE(sumtot) + (ovflow <<3 ));
+   counter = N;  // set the counter to N if iteration should happen right away
 }
 
-#undef MIXMAX_SEED_WAS_ZERO
-
 void MixMaxRng::seed_uniquestream( myID_t clusterID, myID_t machineID, myID_t runID, myID_t  streamID )
 {
-   seed_vielbein(0);
-   S.sumtot = apply_bigskip(S.V.data(), S.V.data(),  clusterID,  machineID,  runID,   streamID );
-   S.counter = 1;
+   state_init();
+   sumtot = apply_bigskip(V, V, clusterID, machineID, runID, streamID );
+   counter = 1;
 }
         
 myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t clusterID, myID_t machineID, myID_t runID, myID_t  streamID )
@@ -626,7 +476,7 @@ myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t cluster
     (this is good enough : a single CPU will not exceed this in the lifetime of the universe, 10^19 sec,
     even if it had a clock cycle of Planch time, 10^44 Hz )
              
-    Caution: never apply this to a derived vector, just choose some mother vector Vin, for example the unit vector by seed_vielbein(X,0),
+    Caution: never apply this to a derived vector, just choose some mother vector Vin, for example the unit vector by state_init(),
     and use it in all your runs, just change runID to get completely nonoverlapping streams of random numbers on a different day.
              
     clusterID and machineID are provided for the benefit of large organizations who wish to ensure that a simulation
@@ -641,7 +491,7 @@ myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t cluster
    ;
             
    const myuint_t* skipMat[128];
-   for (int i=0; i<128; i++) { skipMat[i] = skipMat17[i];}
+   for (int i=0; i<128; ++i) { skipMat[i] = skipMat17[i]; }
             
    myID_t IDvec[4] = {streamID, runID, machineID, clusterID};
    int r,i,j,  IDindex;
@@ -649,10 +499,10 @@ myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t cluster
    myuint_t Y[N], cum[N];
    myuint_t coeff;
    myuint_t* rowPtr;
-   myuint_t sumtot=0;
+   myuint_t insumtot=0;
             
-   for (i=0; i<N; i++) { Y[i] = Vin[i]; sumtot = modadd( sumtot, Vin[i]); } ;
-   for (IDindex=0; IDindex<4; IDindex++)
+   for (i=0; i<N; ++i) { Y[i] = Vin[i]; insumtot = modadd( insumtot, Vin[i]); }
+   for (IDindex=0; IDindex<4; ++IDindex)
    { // go from lower order to higher order ID
      id=IDvec[IDindex];
      //printf("now doing ID at level %d, with ID = %d\n", IDindex, id);
@@ -662,26 +512,26 @@ myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t cluster
        if (id & 1)
        {
          rowPtr = (myuint_t*)skipMat[r + IDindex*8*sizeof(myID_t)];
-         for (i=0; i<N; i++){ cum[i] = 0; }
-         for (j=0; j<N; j++)
+         for (i=0; i<N; ++i){ cum[i] = 0; }
+         for (j=0; j<N; ++j)
          { // j is lag, enumerates terms of the poly
            // for zero lag Y is already given
            coeff = rowPtr[j]; // same coeff for all i
-           for (i =0; i<N; i++){
+           for (i =0; i<N; ++i){
              cum[i] =  fmodmulM61( cum[i], coeff ,  Y[i] ) ;
            }
-           sumtot = iterate_raw_vec(Y, sumtot);
+           insumtot = iterate_raw_vec(Y, insumtot);
          }
-         sumtot=0;
-         for (i=0; i<N; i++){ Y[i] = cum[i]; sumtot = modadd( sumtot, cum[i]); } ;
+         insumtot=0;
+         for (i=0; i<N; ++i){ Y[i] = cum[i]; insumtot = modadd( insumtot, cum[i]); } ;
        }
-       id = (id >> 1); r++; // bring up the r-th bit in the ID
+       id = (id >> 1); ++r; // bring up the r-th bit in the ID
      }
    }
-   sumtot=0;
-   for (i=0; i<N; i++){ Vout[i] = Y[i]; sumtot = modadd( sumtot, Y[i]); }
+   insumtot=0;
+   for (i=0; i<N; ++i){ Vout[i] = Y[i]; insumtot = modadd( insumtot, Y[i]); }
    // returns sumtot, and copy the vector over to Vout
-   return (sumtot) ;
+   return insumtot;
 }
         
 #if defined(__x86_64__)
@@ -714,41 +564,20 @@ myuint_t MixMaxRng::apply_bigskip( myuint_t* Vout, myuint_t* Vin, myID_t cluster
   }
 #endif
 
-myuint_t MixMaxRng::modadd(myuint_t foo, myuint_t bar)
-{
-#if defined(__x86_64__) && defined(__GNUC__) && (!defined(__ICC))
-   //#warning Using assembler routine in modadd
-   myuint_t out;
-   /* Assembler trick suggested by Andrzej Görlich     */
-   __asm__ ("addq %2, %0; "
-            "btrq $61, %0; "
-            "adcq $0, %0; "
-            :"=r"(out)
-            :"0"(foo), "r"(bar)
-           );
-   return out;
-#else
-   return MIXMAX_MOD_MERSENNE(foo+bar);
-#endif
-}
-
 void MixMaxRng::print_state() const
 {
-   int j;
    std::cout << "mixmax state, file version 1.0\n";
    std::cout << "N=" << rng_get_N() << "; V[N]={";
-   for (j=0; (j< (rng_get_N()-1) ); j++) {
-     std::cout << S.V[j] << ", ";
-   }
-   std::cout << S.V[rng_get_N()-1];
+   for (int j=0; (j< (rng_get_N()-1) ); ++j) { std::cout << V[j] << ", "; }
+   std::cout << V[rng_get_N()-1];
    std::cout << "}; ";
-   std::cout << "counter= " << S.counter;
-   std::cout << "sumtot= " << S.sumtot << "\n";
+   std::cout << "counter= " << counter;
+   std::cout << "sumtot= " << sumtot << "\n";
 }
 
 MixMaxRng MixMaxRng::Branch()
 {
-   S.sumtot = iterate_raw_vec(S.V.data(), S.sumtot); S.counter = 1;
+   sumtot = iterate_raw_vec(V, sumtot); counter = 1;
    MixMaxRng tmp=*this;
    tmp.BranchInplace(0); // daughter id
    return tmp;
@@ -759,11 +588,11 @@ void MixMaxRng::BranchInplace(int id)
    // Dont forget to iterate the mother, when branching the daughter, or else will have collisions!
    // a 64-bit LCG from Knuth line 26, is used to mangle a vector component
    constexpr myuint_t MULT64=6364136223846793005ULL;
-   myuint_t tmp=S.V[id];
-   S.V[1] *= MULT64; S.V[id] &= M61;
-   S.sumtot = MIXMAX_MOD_MERSENNE( S.sumtot + S.V[id] - tmp + M61);
-   S.sumtot = iterate_raw_vec(S.V.data(), S.sumtot);// printf("iterating!\n");
-   S.counter = 1;
+   myuint_t tmp=V[id];
+   V[1] *= MULT64; V[id] &= M61;
+   sumtot = MIXMAX_MOD_MERSENNE( sumtot + V[id] - tmp + M61);
+   sumtot = iterate_raw_vec(V, sumtot);
+   counter = 1;
 }
 
 }  // namespace CLHEP
